/* 
 * ProjectDelegate.h created by stoyan on 2003-05-12
 */

#ifndef _PROJECTDELEGATE_H
#define _PROJECTDELEGATE_H

@protocol ProjectDelegate <NSObject>

- (NSDictionary *)projectTypes;

@end

#endif
