/*
   GNUstep ProjectCenter - http://www.gnustep.org

   Copyright (C) 2000-2002 Free Software Foundation

   Author: Philippe C.D. Robert <probert@siggraph.org>

   This file is part of GNUstep.

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _PROJECTCENTER_H
#define _PROJECTCENTER_H

#include <ProjectCenter/PCBundleLoader.h>
#include <ProjectCenter/PCDefines.h>
#include <ProjectCenter/PCProject.h>
#include <ProjectCenter/PCProjectWindow.h>
#include <ProjectCenter/PCProjectBrowser.h>
#include <ProjectCenter/PCProjectLoadedFiles.h>
#include <ProjectCenter/PCProjectEditor.h>
#include <ProjectCenter/PCProjectManager.h>
#include <ProjectCenter/PCProjectBuilder.h>
#include <ProjectCenter/PCProjectLauncher.h>
#include <ProjectCenter/PCProjectEditor.h>
#include <ProjectCenter/PCServer.h>
#include <ProjectCenter/PCFileManager.h>
#include <ProjectCenter/PCEditor.h>
#include <ProjectCenter/PCEditorView.h>
#include <ProjectCenter/PCEditorView+Highlighting.h>
#include <ProjectCenter/PCTextFinder.h>
#include <ProjectCenter/PCMakefileFactory.h>

#include <ProjectCenter/ProjectType.h>
#include <ProjectCenter/Server.h>

#endif

